
GO
PRINT N'Creando [dbo].[ReportFieldAttribute]...';


GO
CREATE TABLE [dbo].[ReportFieldAttribute] (
    [Field]      VARCHAR (50)  NULL,
    [Label]      VARCHAR (50)  NULL,
    [IsDisabled] BIT           NULL,
    [IsDropdown] BIT           NULL,
    [Procedure]  VARCHAR (50)  NULL,
    [Arguments]  VARCHAR (MAX) NULL,
    [Variables]  VARCHAR (MAX) NULL,
    [CreatedAt]  DATETIME      NULL
);


GO
PRINT N'Creando restricción sin nombre en [dbo].[ReportFieldAttribute]...';


GO
ALTER TABLE [dbo].[ReportFieldAttribute]
    ADD DEFAULT ((0)) FOR [IsDisabled];


GO
PRINT N'Creando restricción sin nombre en [dbo].[ReportFieldAttribute]...';


GO
ALTER TABLE [dbo].[ReportFieldAttribute]
    ADD DEFAULT ((0)) FOR [IsDropdown];


GO
PRINT N'Creando restricción sin nombre en [dbo].[ReportFieldAttribute]...';


GO
ALTER TABLE [dbo].[ReportFieldAttribute]
    ADD DEFAULT (getutcdate()) FOR [CreatedAt];


GO
PRINT N'Modificando [dbo].[GetReportProcedureParameters]...';


GO
ALTER PROCEDURE [dbo].[GetReportProcedureParameters](
	@Procedure Varchar(200)
	)
AS
BEGIN
	SET NOCOUNT ON

    DECLARE 
        @object_id INT,
        @paramID INT,
        @paramName SYSNAME,
        @definition NVARCHAR(MAX),
        @t NVARCHAR(MAX),
        @loc1 INT,
        @loc2 INT,
        @loc3 INT,
        @loc4 INT,
        @has_default_value BIT;

    SET @object_id = OBJECT_ID(@Procedure);

    IF (@object_id IS NOT NULL)
    BEGIN
    
        SELECT @definition = OBJECT_DEFINITION(@object_id);

        CREATE TABLE #params
        (
            parameter_id        INT PRIMARY KEY,
            has_default_value    BIT NOT NULL DEFAULT (0)
        );

        DECLARE c CURSOR
        LOCAL FORWARD_ONLY STATIC READ_ONLY
        FOR 
            SELECT 
                parameter_id,
                [name]
            FROM
                sys.parameters
            WHERE
                [object_id] = @object_id;

        OPEN c;

        FETCH NEXT FROM c INTO @paramID, @paramName;

        WHILE (@@FETCH_STATUS = 0)
        BEGIN
    
            SELECT
                @t = SUBSTRING
                (
                    @definition,
                    CHARINDEX(@paramName, @definition),
                    4000
                ),
                @has_default_value = 0;
            
            SET @loc1 = COALESCE(NULLIF(CHARINDEX('''', @t), 0), 4000);
            SET @loc2 = COALESCE(NULLIF(CHARINDEX(',', @t), 0), 4000);
            SET @loc3 = NULLIF(CHARINDEX('OUTPUT', @t), 0);
            SET @loc4 = NULLIF(CHARINDEX('AS', @t), 0);
            
            SET @loc1 = CASE WHEN @loc2 < @loc1 THEN @loc2 ELSE @loc1 END;
            SET @loc1 = CASE WHEN @loc3 < @loc1 THEN @loc3 ELSE @loc1 END;
            SET @loc1 = CASE WHEN @loc4 < @loc1 THEN @loc4 ELSE @loc1 END;
            
            IF CHARINDEX('=', LTRIM(RTRIM(SUBSTRING(@t, 1, @loc1)))) > 0
                SET @has_default_value = 1;

            INSERT #params
            (
                parameter_id,
                has_default_value
            )
            SELECT
                @paramID,
                @has_default_value;
            
            FETCH NEXT FROM c INTO @paramID, @paramName;
        END
        
        SELECT 
           sp.[name]						'parameter',
            --sp.[parameter_id]				'ordinal',
            st.[name]						'type',
           --sp.[max_length]				'maxLength',
           --sp.[precision]					'precission',
           --sp.[scale]						'scale',
           --sp.[is_output]					'isOutput',
            p.[has_default_value]			'hasDefaultValue',
            IIF(RFA.[Label] IS NULL,REPLACE(sp.[name],'@',''),RFA.[Label])               'label',
            IIF(RFA.[IsDisabled] IS NULL,CAST(0 AS BIT),RFA.[IsDisabled])                'isDisabled',
            IIF(RFA.[IsDropdown] IS NULL,CAST(0 AS BIT),RFA.[IsDropdown])                'isDropdown',
            RFA.[Procedure]                 'procedure',
            RFA.[Arguments]                 '_arguments',
            RFA.[Variables]                 'variables'
        FROM
            sys.parameters sp
        INNER JOIN
            #params p
        ON
            sp.parameter_id = p.parameter_id
        INNER JOIN
            sys.types st
        ON
            sp.user_type_id = st.user_type_id
         LEFT JOIN [dbo].[ReportFieldAttribute] RFA (NOLOCK)
         ON RFA.[Field] =  sp.[name]
         
        WHERE
            sp.[object_id] = @object_id;
        
        CLOSE c;
        DEALLOCATE c;
        DROP TABLE #params;
    END
END
GO
PRINT N'Modificando [dbo].[ReportByAccountSubAccountCharge]...';


GO
ALTER PROCEDURE [dbo].[ReportByAccountSubAccountCharge] (
	@PlatformHash VARCHAR(100)=NULL,
	@StartDate DATETIME = NULL,
	@EndDate DATETIME = NULL,
	@InstallationId VARCHAR(100),
	@AccountId VARCHAR(50),
	@SubAccountId VARCHAR(50) = NULL
	)
AS
BEGIN
	-- ESTE PROCEDURE TIENE POR FINALIDAD RETORNAR EL REPORTE DE RECARGA DE SALDOS
	SET NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
	
		-- EN EL CASO SUBACCOUNT SEA NULL
		IF @SubAccountId IS NULL
		BEGIN

			WITH CTE_CHARGE_REPORT AS (
				SELECT ASAVBM.[TimeStamp]											'COLUMN_DATE',
						A.[InstallationId]											'COLUMN_PLATFORM',
						A.[AccountId]												'COLUMN_ACCOUNT',
						A.[SubAccountId]											'COLUMN_SUBACCOUNT',
						C.[Code]													'COLUMN_CURRENCY_CODE',
						C.[Symbol]													'COLUMN_CURRENCY_SYMBOL',
						ASAVBM.[CreatedByUserId]									'COLUMN_CREATED_BY',	
						ASAVBM.[PrevCountableBalance]								'COLUMN_PREVIOUS_COUNTABLE_BALANCE',
						ASAVBM.[Amount]												'COUMNN_AMOUNT',
						A.[TranscriptionRatePerHour]								'COLUMN_TRANSCRIPTION_RATE_PER_HOUR',
						-- A.[TranscriptionRatePerSecond]								'COLUMN_TRANSCRIPTION_RATE_PER_SECOND',
						A.[SemanticAnalysisRatePerTransaction]						'COLUMN_SEMANTIC_ANALYSIS_RATE_PER_TRANSACTION',
						A.[CognitiveAnalysisRatePerTransaction]						'COLUMN_COGNITIVE_ANALYSIS_RATE_PER_TRANSACTION',
						STUFF((SELECT ', ' + AV.[VCC]
								FROM [dbo].[AccountVCC] AV (NOLOCK)
								WHERE AV.[InstallationId] = A.[InstallationId]
									AND AV.[AccountId] = A.[AccountId]
									AND AV.[SubAccountId] IS NULL
								ORDER BY AV.[VCC]
								FOR XML PATH('')), 1, 1, '' ) AS					'COLUMN_RELATED_VCCS'
					FROM [dbo].[Account] A (NOLOCK)
					LEFT JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
						ON ASAVBM.[InstallationId] = A.[InstallationId]
							AND ASAVBM.[AccountId] = A.[AccountId]
							AND ASAVBM.[SubAccountId] IS NULL
							AND ASAVBM.[VCC]  IS NULL 
							AND ASAVBM.[Wallet] = 'FREE'
							AND ASAVBM.[Type] = 'CHARGE'
					INNER JOIN [dbo].[Currency] C (NOLOCK)
						ON C.[Code] = ASAVBM.[CurrencyCode]
					WHERE A.[InstallationId] = @InstallationId
						AND A.[AccountId] = @AccountId
						AND A.[SubAccountId] IS NULL
						AND ASAVBM.[TimeStamp] BETWEEN @StartDate AND @EndDate 
				UNION
				SELECT ASAVBM.[TimeStamp]											'COLUMN_DATE',
						A.[InstallationId]											'COLUMN_PLATFORM',
						A.[AccountId]												'COLUMN_ACCOUNT',
						A.[SubAccountId]											'COLUMN_SUBACCOUNT',
						C.[Code]													'COLUMN_CURRENCY_CODE',
						C.[Symbol]													'COLUMN_CURRENCY_SYMBOL',
						ASAVBM.[CreatedByUserId]									'COLUMN_CREATED_BY',	
						ASAVBM.[PrevCountableBalance]								'COLUMN_PREVIOUS_COUNTABLE_BALANCE',
						ASAVBM.[Amount]												'COUMNN_AMOUNT',
						A.[TranscriptionRatePerHour]								'COLUMN_TRANSCRIPTION_RATE_PER_HOUR',
						-- A.[TranscriptionRatePerSecond]								'COLUMN_TRANSCRIPTION_RATE_PER_SECOND',
						A.[SemanticAnalysisRatePerTransaction]						'COLUMN_SEMANTIC_ANALYSIS_RATE_PER_TRANSACTION',
						A.[CognitiveAnalysisRatePerTransaction]						'COLUMN_COGNITIVE_ANALYSIS_RATE_PER_TRANSACTION',
						STUFF((SELECT ', ' + AV.[VCC]
								FROM [dbo].[AccountVCC] AV (NOLOCK)
								WHERE AV.[InstallationId] = A.[InstallationId]
									AND AV.[AccountId] = A.[AccountId]
									AND AV.[SubAccountId] = A.[SubAccountId]
								ORDER BY AV.[VCC]
								FOR XML PATH('')), 1, 1, '' ) AS					'COLUMN_RELATED_VCCS'
					FROM [dbo].[Account] A (NOLOCK)
					LEFT JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
						ON ASAVBM.[InstallationId] = A.[InstallationId]
							AND ASAVBM.[AccountId] = A.[AccountId]
							AND ASAVBM.[SubAccountId] = A.[SubAccountId]
							AND ASAVBM.[VCC]  IS NULL 
							AND ASAVBM.[Wallet] = 'FREE'
							AND ASAVBM.[Type] = 'CHARGE'
					INNER JOIN [dbo].[Currency] C (NOLOCK)
						ON C.[Code] = ASAVBM.[CurrencyCode]
					WHERE A.[InstallationId] = @InstallationId
						AND A.[AccountId] = @AccountId
						AND A.[SubAccountId] IS NOT NULL
						AND ASAVBM.[TimeStamp] BETWEEN @StartDate AND @EndDate 
			)

			-- EL CTE SOLO PUEDE SER INVOCADO UNA VEZ POR ESO LO PONEMOS EN EL SELECT FINAL DONDE EL CONTADOR(RN) ES 1
			SELECT * 
				FROM CTE_CHARGE_REPORT
				ORDER BY [COLUMN_DATE] DESC

		END
		ELSE
		-- EN EL CASO SUBACCOUNT LLEGUE CON UN VALOR
		BEGIN
			SELECT ASAVBM.[TimeStamp]											'COLUMN_DATE',
					A.[InstallationId]											'COLUMN_PLATFORM',
					A.[AccountId]												'COLUMN_ACCOUNT',
					A.[SubAccountId]											'COLUMN_SUBACCOUNT',
					C.[Code]													'COLUMN_CURRENCY_CODE',
					C.[Symbol]													'COLUMN_CURRENCY_SYMBOL',
					ASAVBM.[CreatedByUserId]									'COLUMN_CREATED_BY',	
					ASAVBM.[PrevCountableBalance]								'COLUMN_PREVIOUS_COUNTABLE_BALANCE',
					ASAVBM.[Amount]												'COUMNN_AMOUNT',
					A.[TranscriptionRatePerHour]								'COLUMN_TRANSCRIPTION_RATE_PER_HOUR',
					-- A.[TranscriptionRatePerSecond]								'COLUMN_TRANSCRIPTION_RATE_PER_SECOND',
					A.[SemanticAnalysisRatePerTransaction]						'COLUMN_SEMANTIC_ANALYSIS_RATE_PER_TRANSACTION',
					A.[CognitiveAnalysisRatePerTransaction]						'COLUMN_COGNITIVE_ANALYSIS_RATE_PER_TRANSACTION',
					STUFF((SELECT ', ' + AV.[VCC]
							FROM [dbo].[AccountVCC] AV (NOLOCK)
							WHERE AV.[InstallationId] = A.[InstallationId]
								AND AV.[AccountId] = A.[AccountId]
								AND AV.[SubAccountId] = A.[SubAccountId]
							ORDER BY AV.[VCC]
							FOR XML PATH('')), 1, 1, '' ) AS					'COLUMN_RELATED_VCCS'
				FROM [dbo].[Account] A (NOLOCK)
				LEFT JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
					ON ASAVBM.[InstallationId] = A.[InstallationId]
						AND ASAVBM.[AccountId] = A.[AccountId]
						AND ASAVBM.[SubAccountId]  = A.[SubAccountId]
						AND ASAVBM.[VCC]  IS NULL 
						AND ASAVBM.[Wallet] = 'FREE'
						AND ASAVBM.[Type] = 'CHARGE'
				INNER JOIN [dbo].[Currency] C (NOLOCK)
					ON C.[Code] = ASAVBM.[CurrencyCode]
				WHERE A.[InstallationId] = @InstallationId
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] = @SubAccountId 
					AND ASAVBM.[TimeStamp] BETWEEN @StartDate AND @EndDate 
				ORDER BY ASAVBM.[TimeStamp] DESC

		END 

	END TRY
    BEGIN CATCH
        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1

    END CATCH

	SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[ReportByVCCBalanceDetail]...';


GO
ALTER PROCEDURE [dbo].[ReportByVCCBalanceDetail] (
	@PlatformHash VARCHAR(100)=NULL,
	@StartDate DATETIME,
	@EndDate DATETIME,
	@InstallationId VARCHAR(100)=NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@UserApi VARCHAR(50) = NULL,
	@BatchTypeName VARCHAR(50) = NULL,
	@BatchTypeId VARCHAR(50) = NULL	
	)
AS
BEGIN
	-- ESTE PROCEDURE TIENE POR FINALIDAD HACER UN REPORTE DETALLADO DE TRANSACCIONES
	SET NOCOUNT ON

	-- EN CASO EL VCC SEA SYSTEM LO IGUALAMOS A NULL
	IF @VCC = 'system' SET @VCC = NULL

	SELECT
		D.[ClientId]													'COLUMN_CLIENT_NAME',
		S.[StartDate]													'COLUMN_START_DATE',
		S.[EndDate]														'COLUMN_END_DATE',
		D.[BatchTypeName]												'COLUMN_BATCH_TYPE_NAME',
		D.[BatchTypeId]													'COLUMN_BATCH_TYPE_ID',
		D.[AccountId]													'COLUMN_ACCOUNT',
		D.[SubAccountId]												'COLUMN_SUBACCOUNT',
		C.[Code]														'COLUMN_CURRENCY_CODE',
		-- C.[Symbol]														'COLUMN_CURRENCY_SYMBOL',
		D.[ApiId]														'COLUMN_API_ID',
		D.[SystemService]												'COLUMN_SYSTEM_SERVICE',
		CASE WHEN D.[SystemService] = 'TRANSCRIPTION' 
			THEN SUM(ISNULL(D.[Rate], 0)) ELSE NULL END					'COLUMN_TRANSCRIPTION_TOTAL_RATE',
		D.[RatePerSecond]												'COLUMN_TRANSCRIPTION_RATE_PER_SECOND',
		SUM(ISNULL(D.[ValidDurationTime], 0))							'COLUMN_TRANSCRIPTION_VALID_DURATION_TIME',
		S.[SemanticAnalysisRatePerTransaction]							'COLUMN_SEMANTIC_ANALYSIS_RATE_PER_TRANSACTION',
		CASE WHEN S.[TotalQueries] > 0 THEN 1 ELSE NULL END				'COLUMN_SEMANTIC_ANALYSIS_TOTAL_REQUEST',
		CASE WHEN D.[SystemService] = 'SEMANTIC_ANALYSIS' 
			THEN SUM(ISNULL(D.[Rate], 0)) ELSE NULL END					'COLUMN_SEMANTIC_ANALYSIS_TOTAL_RATE',
		S.[CognitiveAnalysisRatePerTransaction]							'COLUMN_COGNITIVE_ANALYSIS_RATE_PER_TRANSACTION',
		CASE WHEN D.[SystemService] = 'COGNITIVE_ANALYSIS' 
			THEN SUM(D.[TotalValidRequest]) ELSE NULL END				'COLUMN_COGNITIVE_ANALYSIS_TOTAL_REQUEST',
		CASE WHEN D.[SystemService] = 'COGNITIVE_ANALYSIS' 
			THEN SUM(ISNULL(D.[Rate], 0)) ELSE NULL END					'COLUMN_COGNITIVE_ANALYSIS_TOTAL_RATE',
		D.[EntityId]													'COLUMN_ENTITY_ID'
	FROM AccountSubAccountVCCDetailedTransaction D (NOLOCK)
	LEFT JOIN AccountSubAccountVCCSummaryTransaction S (NOLOCK)
		ON S.BatchID = D.BatchId
			AND S.[SystemService] = D.[SystemService]
	INNER JOIN [dbo].[Currency] C (NOLOCK)
		ON C.[Code] = S.[CurrencyCode]
	WHERE D.[InstallationId] IS NOT NULL
		AND   (@InstallationId IS NULL OR D.[InstallationId] = @InstallationId)
		AND   (@StartDate IS NULL OR S.[EndDate] >= @StartDate)
		AND   (@EndDate IS NULL OR S.[EndDate] <= @EndDate)
		AND   (@BatchTypeName IS NULL OR D.[BatchTypeName] = @BatchTypeName)
		AND   (@BatchTypeId IS NULL OR D.[BatchTypeId] = @BatchTypeId)
		AND   (@AccountId IS NULL OR D.[AccountId] = @AccountId)
		AND   (
				((@SubAccountId IS NULL OR @SubAccountId = 'NONE') AND D.[SubAccountId] IS NULL)
				OR (@SubAccountId = 'ALL_SUBACCOUNTS' AND D.[SubAccountId] IS NOT NULL)
				OR D.[SubAccountId] = @SubAccountId
		)
		AND   (
				(@VCC IS NOT NULL AND D.[VCC] = @VCC)
				OR D.[VCC] IS NOT NULL
			)
		AND   (@UserApi IS NULL OR S.[UserApi] = @UserApi)
	GROUP BY D.[ClientId], 
				S.[StartDate], 
				S.[EndDate], 
				D.[BatchTypeName], 
				D.[BatchTypeId], 
				D.[AccountId], 
				D.[SubAccountId], 
				C.[Code], 
				-- C.[Symbol], 
				D.[ApiId], 
				D.[SystemService], 
				D.[RatePerSecond], 
				S.[SemanticAnalysisRatePerTransaction], 
				S.[TotalQueries], 
				S.[CognitiveAnalysisRatePerTransaction], 
				S.[TotalModelRequest], 
				D.[EntityId]

	SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[ReportSummaryofExpensesbyPeriod]...';


GO
ALTER Procedure [dbo].[ReportSummaryofExpensesbyPeriod] (
	@StartDate DATETIME,
	@EndDate DATETIME,
	@InstallationId Varchar(150),
	@AccountId Varchar(150),
	@SubAccountId Varchar(150)  = NULL,
	@VCC Varchar(150) = NULL,
	@Slice Int = 1
) As
BEGIN 
	-- EN CASO EL VALOR DE VCC LEGUE COMO SYSTEM LO IGUALAMOS A NULL
	IF @VCC = 'system' SET @VCC = NULL

	SELECT 
	  case @Slice
		when 1 then Dateadd(DAY,datediff(DAY, 0, A.enddate), 0)
		when 30 then Dateadd(MONTH,datediff(MONTH, 0, A.enddate), 0)
		else Dateadd(DAY,datediff(DAY, 0, A.enddate), 0)
	  end																											AS 'COLUMN_SLICE',
	  --Dateadd(day, 0, @Slice * Floor(Datediff(day, 0, A.enddate) / @Slice)) AS SLICE, 
	  A.systemservice																								AS 'COLUMN_BATCH_TYPE_NAME', 
	  Avg(A.TranscriptionRatePerSecond)																				AS 'COLUMN_TRANSCRIPTION_RATE_PER_SECOND', 
	  avg(A.SemanticAnalysisRatePerTransaction)																		AS 'COLUMN_SEMANTIC_ANALYSIS_RATE_PER_TRANSACTION', 
	  avg(A.CognitiveAnalysisRatePerTransaction)																	AS 'COLUMN_COGNITIVE_ANALYSIS_RATE_PER_TRANSACTION', 
	  A.installationid																								AS 'COLUMN_PLATFORM', 
	  B.accountid																									AS 'COLUMN_ACCOUNT', 
	  B.subaccountid																								AS 'COLUMN_SUBACCOUNT', 
	  C.[Code]																										AS 'COLUMN_CURRENCY_CODE',
	  -- C.[Symbol]																									AS 'COLUMN_CURRENCY_SYMBOL',
	  A.vcc																											AS 'COLUMN_VCC', 
	  Sum(CASE WHEN A.[SystemService] = 'TRANSCRIPTION' THEN ISNULL(A.[ValidDurationTime], 0) ELSE 0 END)			AS 'COLUMN_TRANSCRIPTION_VALID_DURATION_TIME', 
	  -- Sum(A.Totalqueries) AS CANTIDADAS, 
	  Sum(A.TotalQueryRequest)																						AS 'COLUMN_SEMANTIC_ANALYSIS_TOTAL_REQUEST', 
	  Sum(A.totalmodelrequest)																						AS 'COLUMN_COGNITIVE_ANALYSIS_TOTAL_REQUEST', 
	  --Sum(A.Rate) As Rate, 
	  SUM(CASE WHEN A.[SystemService] = 'TRANSCRIPTION' THEN ISNULL(A.[Rate], 0) ELSE 0 END)						AS 'COLUMN_TRANSCRIPTION_TOTAL_RATE', 
	  SUM(CASE WHEN A.[SystemService] = 'SEMANTIC_ANALYSIS' THEN ISNULL(A.[Rate], 0) ELSE 0 END)					AS 'COLUMN_SEMANTIC_ANALYSIS_TOTAL_RATE', 
	  SUM(CASE WHEN A.[SystemService] = 'COGNITIVE_ANALYSIS' THEN ISNULL(A.[Rate], 0) ELSE 0 END)					AS 'COLUMN_COGNITIVE_ANALYSIS_TOTAL_RATE' 
	  /*
	  Sum(A.transcriptionratepersecond) * Sum(CASE WHEN A.[SystemService] = 'TRANSCRIPTION' THEN ISNULL(A.[ValidDurationTime], 0) ELSE 0 END) AS TOTALTR, 
	  Sum(A.semanticanalysisratepertransaction) * Sum(A.validtotalqueries) AS TOTALAS, 
	  Sum(A.cognitiveanalysisratepertransaction) * Sum(A.totalmodelrequest) AS TOTALAC */
	FROM accountsubaccountvccsummarytransaction (nolock) AS A 
	LEFT JOIN accountvcc (nolock) AS B 
		ON A.installationid = B.installationid 
			AND A.vcc = B.vcc 
	INNER JOIN [dbo].[Currency] C (NOLOCK)
		ON C.[Code] = A.[CurrencyCode]
	WHERE A.installationid = @InstallationId 
		AND A.enddate BETWEEN @StartDate 
		AND @EndDate 
		AND B.accountid LIKE CASE WHEN @AccountId IS NOT NULL THEN @AccountId ELSE B.accountid END 
		AND   (
			((@SubAccountId IS NULL OR @SubAccountId = 'NONE') AND B.[SubAccountId] IS NULL)
			OR (@SubAccountId = 'ALL_SUBACCOUNTS' AND B.[SubAccountId] IS NOT NULL)
			OR B.[SubAccountId] = @SubAccountId
		)
		AND   (
			(@VCC IS NOT NULL AND A.[VCC] = @VCC)
			OR A.[VCC] IS NOT NULL
		)
		--AND A.vcc LIKE CASE WHEN @VCC IS NOT NULL THEN @VCC ELSE A.vcc END -- 
	GROUP BY 
	  case @Slice
		when 1 then Dateadd(DAY,datediff(DAY, 0, A.enddate), 0)
		when 30 then Dateadd(MONTH,datediff(MONTH, 0, A.enddate), 0)
		else Dateadd(DAY,datediff(DAY, 0, A.enddate), 0)
	  end,
	  --Dateadd(DAY,datediff(DAY, 0, A.enddate), 0),
	  --Dateadd(day, 0, @Slice * Floor(Datediff(day, 0, A.enddate) / @Slice)), 
	  A.vcc, 
	  A.systemservice, 
	  B.accountid, 
	  B.subaccountid, 
	  C.[Code],
	  -- C.[Symbol],
	  A.installationid

END
GO
PRINT N'Actualización completada.';


GO




GO
PRINT N'Modificando [dbo].[UpdatePlatformHash]...';


GO
ALTER PROCEDURE [dbo].[UpdatePlatformHash] (
	@PlatformHash VARCHAR(100) = NULL,
	@PlatformHashNew VARCHAR(100) = NULL,
	@User VARCHAR(50) = NULL
	)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS PLATFORMHASH
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

			--Actualizamos en la  tabla Account
			UPDATE [Account]
			SET [PlatformHash]=@PlatformHashNew
			WHERE [PlatformHash]=@PlatformHash

			--Actualizamos Account Balance
			UPDATE[AccountBalance]
			SET [PlatformHash]=@PlatformHashNew
			WHERE [PlatformHash]=@PlatformHash

			/*
			--Actualizamo AccounSubAccountVCCDetailedBalance
			UPDATE[AccountSubAccountVCCDetailedBalance]
			SET [PlatformHash]=@PlatformHashNew
			WHERE [PlatformHash]=@PlatformHash
			
			--Actualizamos AccountSubAccountVCCGeneralBalance
			UPDATE[AccountSubAccountVCCGeneralBalance]
			SET [PlatformHash]=@PlatformHashNew
			WHERE [PlatformHash]=@PlatformHash
			*/

			--Actualizamos Platform
			UPDATE[Platform]
			SET [PlatformHash]=@PlatformHashNew
			WHERE [PlatformHash]=@PlatformHash

			--ActualizamosPlatformData
			UPDATE[PlatformData]
			SET [PlatformHash]=@PlatformHashNew
			WHERE [PlatformHash]=@PlatformHash

			--Actualizamos PlatformLicense
			UPDATE[PlatformLicense]
			SET [PlatformHash]=@PlatformHashNew
			WHERE [PlatformHash]=@PlatformHash


			SELECT 1 AS 'result'
		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION
	END TRY

    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
END
GO
PRINT N'Actualización completada.';


GO

