
GO
PRINT N'Modificando [dbo].[GetReportProcedureParameters]...';


GO
ALTER PROCEDURE [dbo].[GetReportProcedureParameters](
	@Procedure Varchar(200)
	)
AS
BEGIN
	SET NOCOUNT ON

    DECLARE 
        @object_id INT,
        @paramID INT,
        @paramName SYSNAME,
        @definition NVARCHAR(MAX),
        @t NVARCHAR(MAX),
        @loc1 INT,
        @loc2 INT,
        @loc3 INT,
        @loc4 INT,
        @has_default_value BIT;

    SET @object_id = OBJECT_ID(@Procedure);

    IF (@object_id IS NOT NULL)
    BEGIN
    
        SELECT @definition = OBJECT_DEFINITION(@object_id);

        CREATE TABLE #params
        (
            parameter_id        INT PRIMARY KEY,
            has_default_value    BIT NOT NULL DEFAULT (0)
        );

        DECLARE c CURSOR
        LOCAL FORWARD_ONLY STATIC READ_ONLY
        FOR 
            SELECT 
                parameter_id,
                [name]
            FROM
                sys.parameters
            WHERE
                [object_id] = @object_id;

        OPEN c;

        FETCH NEXT FROM c INTO @paramID, @paramName;

        WHILE (@@FETCH_STATUS = 0)
        BEGIN
    
            SELECT
                @t = SUBSTRING
                (
                    @definition,
                    CHARINDEX(@paramName, @definition),
                    4000
                ),
                @has_default_value = 0;
            
            SET @loc1 = COALESCE(NULLIF(CHARINDEX('''', @t), 0), 4000);
            SET @loc2 = COALESCE(NULLIF(CHARINDEX(',', @t), 0), 4000);
            SET @loc3 = NULLIF(CHARINDEX('OUTPUT', @t), 0);
            SET @loc4 = NULLIF(CHARINDEX('AS', @t), 0);
            
            SET @loc1 = CASE WHEN @loc2 < @loc1 THEN @loc2 ELSE @loc1 END;
            SET @loc1 = CASE WHEN @loc3 < @loc1 THEN @loc3 ELSE @loc1 END;
            SET @loc1 = CASE WHEN @loc4 < @loc1 THEN @loc4 ELSE @loc1 END;
            
            IF CHARINDEX('=', LTRIM(RTRIM(SUBSTRING(@t, 1, @loc1)))) > 0
                SET @has_default_value = 1;

            INSERT #params
            (
                parameter_id,
                has_default_value
            )
            SELECT
                @paramID,
                @has_default_value;
            
            FETCH NEXT FROM c INTO @paramID, @paramName;
        END
        
        --CONSULTAR SI EXISTE EN LA BASE DE DATOS LA TABLA ReportFieldAttrubute
        IF OBJECT_ID('dbo.ReportFieldAttribute') IS NOT NULL
        BEGIN
		        SELECT 
		           sp.[name]						'parameter',
		            st.[name]						'type',
		            p.[has_default_value]			'hasDefaultValue',
		            IIF(RFA.[Label] IS NULL,REPLACE(sp.[name],'@',''),RFA.[Label])               'label',
		            IIF(RFA.[IsDisabled] IS NULL,CAST(0 AS BIT),RFA.[IsDisabled])                'isDisabled',
		            IIF(RFA.[IsDropdown] IS NULL,CAST(0 AS BIT),RFA.[IsDropdown])                'isDropdown',
		            RFA.[Procedure]                 'procedure',
		            RFA.[Arguments]                 '_arguments',
		            RFA.[Variables]                 'variables'
		        FROM
		            sys.parameters sp
		        INNER JOIN
		            #params p
		        ON
		            sp.parameter_id = p.parameter_id
		        INNER JOIN
		            sys.types st
		        ON
		            sp.user_type_id = st.user_type_id
		         LEFT JOIN [dbo].[ReportFieldAttribute] RFA (NOLOCK)
		         ON RFA.[Field] =  sp.[name]
		         
		        WHERE
		            sp.[object_id] = @object_id;
        END
        ELSE
        BEGIN
        SELECT 
           sp.[name]						'parameter',
            --sp.[parameter_id]				'ordinal',
            st.[name]						'type',
           --sp.[max_length]				'maxLength',
           --sp.[precision]					'precission',
           --sp.[scale]						'scale',
           --sp.[is_output]					'isOutput',
            p.[has_default_value]			'hasDefaultValue',
            REPLACE(sp.[name],'@','')       'label',
            CAST(0 AS BIT)                  'isDisabled',
            CAST(0 AS BIT)                  'isDropdown',
            NULL                            'procedure',
            NULL                            '_arguments',
            NULL                            'variables'
        FROM
            sys.parameters sp
        INNER JOIN
            #params p
        ON
            sp.parameter_id = p.parameter_id
        INNER JOIN
            sys.types st
        ON
            sp.user_type_id = st.user_type_id
         LEFT JOIN [dbo].[ReportFieldAttribute] RFA (NOLOCK)
         ON RFA.[Field] =  sp.[name]
         
        WHERE
            sp.[object_id] = @object_id;
        END
        
        
        
        CLOSE c;
        DEALLOCATE c;
        DROP TABLE #params;
    END
END
GO
PRINT N'Modificando [dbo].[ReportSummaryofExpensesbyPeriod]...';


GO
ALTER Procedure [dbo].[ReportSummaryofExpensesbyPeriod] (
	@StartDate DATETIME,
	@EndDate DATETIME,
	@InstallationId Varchar(150),
	@AccountId Varchar(150),
	@SubAccountId Varchar(150)  = NULL,
	@VCC Varchar(150) = NULL,
	@Slice Int = 1
) As
BEGIN 
	-- EN CASO EL VALOR DE VCC LLEGUE COMO SYSTEM LO IGUALAMOS A NULL
	IF @VCC = 'system' SET @VCC = NULL

	SELECT 
	  case @Slice
		when 1 then Dateadd(DAY,datediff(DAY, 0, A.enddate), 0)
		when 30 then Dateadd(MONTH,datediff(MONTH, 0, A.enddate), 0)
		else Dateadd(DAY,datediff(DAY, 0, A.enddate), 0)
	  end																											AS 'COLUMN_SLICE',
	  --Dateadd(day, 0, @Slice * Floor(Datediff(day, 0, A.enddate) / @Slice)) AS SLICE, 
	  A.systemservice																								AS 'COLUMN_BATCH_TYPE_NAME', 
	  Avg(A.TranscriptionRatePerSecond)																				AS 'COLUMN_TRANSCRIPTION_RATE_PER_SECOND', 
	  avg(A.SemanticAnalysisRatePerTransaction)																		AS 'COLUMN_SEMANTIC_ANALYSIS_RATE_PER_TRANSACTION', 
	  avg(A.CognitiveAnalysisRatePerTransaction)																	AS 'COLUMN_COGNITIVE_ANALYSIS_RATE_PER_TRANSACTION', 
	  A.installationid																								AS 'COLUMN_PLATFORM', 
	  B.accountid																									AS 'COLUMN_ACCOUNT', 
	  B.subaccountid																								AS 'COLUMN_SUBACCOUNT', 
	  C.[Code]																										AS 'COLUMN_CURRENCY_CODE',
	  -- C.[Symbol]																									AS 'COLUMN_CURRENCY_SYMBOL',
	  A.vcc																											AS 'COLUMN_VCC', 
	  Sum(CASE WHEN A.[SystemService] = 'TRANSCRIPTION' THEN ISNULL(A.[ValidDurationTime], 0) ELSE 0 END)			AS 'COLUMN_TRANSCRIPTION_VALID_DURATION_TIME', 
	  -- Sum(A.Totalqueries) AS CANTIDADAS, 
	  Sum(A.TotalQueryRequest)																						AS 'COLUMN_SEMANTIC_ANALYSIS_TOTAL_REQUEST', 
	  Sum(A.totalmodelrequest)																						AS 'COLUMN_COGNITIVE_ANALYSIS_TOTAL_REQUEST', 
	  --Sum(A.Rate) As Rate, 
	  SUM(CASE WHEN A.[SystemService] = 'TRANSCRIPTION' THEN ISNULL(A.[Rate], 0) ELSE 0 END)						AS 'COLUMN_TRANSCRIPTION_TOTAL_RATE', 
	  SUM(CASE WHEN A.[SystemService] = 'SEMANTIC_ANALYSIS' THEN ISNULL(A.[Rate], 0) ELSE 0 END)					AS 'COLUMN_SEMANTIC_ANALYSIS_TOTAL_RATE', 
	  SUM(CASE WHEN A.[SystemService] = 'COGNITIVE_ANALYSIS' THEN ISNULL(A.[Rate], 0) ELSE 0 END)					AS 'COLUMN_COGNITIVE_ANALYSIS_TOTAL_RATE' 
	  /*
	  Sum(A.transcriptionratepersecond) * Sum(CASE WHEN A.[SystemService] = 'TRANSCRIPTION' THEN ISNULL(A.[ValidDurationTime], 0) ELSE 0 END) AS TOTALTR, 
	  Sum(A.semanticanalysisratepertransaction) * Sum(A.validtotalqueries) AS TOTALAS, 
	  Sum(A.cognitiveanalysisratepertransaction) * Sum(A.totalmodelrequest) AS TOTALAC */
	FROM accountsubaccountvccsummarytransaction (nolock) AS A 
	LEFT JOIN accountvcc (nolock) AS B 
		ON A.installationid = B.installationid 
			AND A.vcc = B.vcc 
	INNER JOIN [dbo].[Currency] C (NOLOCK)
		ON C.[Code] = A.[CurrencyCode]
	WHERE A.installationid = @InstallationId 
		AND A.enddate BETWEEN @StartDate 
		AND @EndDate 
		AND B.accountid LIKE CASE WHEN @AccountId IS NOT NULL THEN @AccountId ELSE B.accountid END 
		AND   (
			((@SubAccountId IS NULL OR @SubAccountId = 'NONE') AND B.[SubAccountId] IS NULL)
			OR (@SubAccountId = 'ALL_SUBACCOUNTS' AND B.[SubAccountId] IS NOT NULL)
			OR B.[SubAccountId] = @SubAccountId
		)
		AND   (
			(@VCC IS NOT NULL AND A.[VCC] = @VCC)
			OR A.[VCC] IS NOT NULL
		)
		--AND A.vcc LIKE CASE WHEN @VCC IS NOT NULL THEN @VCC ELSE A.vcc END -- 
	GROUP BY 
	  case @Slice
		when 1 then Dateadd(DAY,datediff(DAY, 0, A.enddate), 0)
		when 30 then Dateadd(MONTH,datediff(MONTH, 0, A.enddate), 0)
		else Dateadd(DAY,datediff(DAY, 0, A.enddate), 0)
	  end,
	  --Dateadd(DAY,datediff(DAY, 0, A.enddate), 0),
	  --Dateadd(day, 0, @Slice * Floor(Datediff(day, 0, A.enddate) / @Slice)), 
	  A.vcc, 
	  A.systemservice, 
	  B.accountid, 
	  B.subaccountid, 
	  C.[Code],
	  -- C.[Symbol],
	  A.installationid

END
GO
PRINT N'Actualización completada.';


GO
